/*--------------------------Rotating Gate 001.c  -----------------------------------
 * 
 * 
 * Hardware: ATMega 328p mit 8MHz internal Clock
 * Board: Rotating Gate 090-F and 090-B 
 * 
 * 
 * avrdude -p m328p -c avrisp2                      ; Console
 * avrdude: Device signature = 0x1e950f             ; default output
 * avrdude: safemode: Fuses OK (E:07, H:D9, L:62)   ; default output
 * 
 * 
 * 
 * Fuse setting:
 * avrdude -p m328p -c avrisp2 -U lfuse:w:0xe2:m 
 * avrdude -p m328p -c avrisp2 -U hfuse:w:0xd9:m 
 * 
 * 
 * 
 * compile:
 * avr-gcc -Wall -Os -mmcu=atmega328p Rotating_Gate_001.c -o Rotating_Gate_001.elf
 * avr-objcopy -O ihex -j .text -j .data Rotating_Gate_001.elf Rotating_Gate_001.hex
 * avrdude -p m328p -c avrisp2 -U flash:w:Rotating_Gate_001.hex
 * 
 * 
 * ADC0: Steps potentiometer
 * ADC1: Mode potentiometer
 * INT0: Rising edge
 * INT1: Falling edge
 * PD7: Gate in LED
 * PB0 - PB7: Gate outputs
 * 
 * 28.04.21
 * Start
 * 
 * ---------------------------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>


/* ------------ Variables --------------------------*/

volatile uint8_t max_steps;				// Holds maximum used steps derived from ADC0
volatile uint8_t steps_read;			// Holds result from ADC0 steps potentiometer
volatile uint8_t count_steps;			// Counts done steps
volatile uint8_t mode;						// Holds mode derived from ADC1; 1=cycle up
																	// Currently only one mode is implemented


uint8_t GateOut[] = 
    { 
0b00000000, 
0b00000001,
0b00000010,
0b00000100,
0b00001000,
0b00010000,
0b00100000,
0b01000000,
0b10000000
    };

/* Interrupt vectors */

//INT0 interrupt 
ISR(INT0_vect )                         // Clock in goes high
    
    { 
				if (max_steps == 0)							// All Gates stay off
					PORTD = 0b10000000;						// Gate LED on
				else 
				{	
					
					PORTB = GateOut[count_steps];	  // Gate out high. Start with GateOut[1]
					PORTD = 0b10000000;							// Gate LED on
				}
				if (count_steps >= max_steps)   // Cycle through array until max_steps reached
					count_steps = 0;							// Start again on top
    } 

ISR(INT1_vect)              // Clock in goes low
{
    PORTB = 0b00000000;     // All Gate outputs to zero
    PORTD = 0b00000000;			// Gate LED off
    if (max_steps > 0)
			count_steps++;				// increase clock counter
}
    
/* ADC init */

void ADC_Init(void) {
 
  uint16_t result;
 
  ADMUX |= (1 << REFS0);					// reference voltage on AVCC    
	ADMUX |= (1 << ADLAR);					// left adjust result, return only 8bits
  // Bit ADFR ("free running") in ADCSRA is already set to zero on startup; single conversion
  ADCSRA |= (1<<ADPS2);    				// Prescaler 8MHz / 16 = 250kHz empf: 50 - 200 kHz
  ADCSRA |= (1<<ADEN);            // ADC activate
  ADCSRA |= (1<<ADSC);						// eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC) )			// Wait for end of conversion
	{         	
  }
  result = ADCW;									//ADCW must be read once
}


/* ADC single measure */
uint8_t ADC_Read( uint8_t channel )
{
  // Choose channnel
  ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
  ADCSRA |= (1<<ADSC);            // single conversion
  while (ADCSRA & (1<<ADSC) ) 		// Wait for end of conversion
	{   
  }
  return (ADCH);           		    // ADC High byte auslesen und zurückgeben
}                             
  
      

/* ----------------------MAIN ---------------------------*/
int main (void)
{					
  DDRD &=~ (1 << PD3);         	  // set PD3 as input; INT0  
  DDRD &=~ (1 << PD2);            // set PD2 as input; INT1  
	DDRD |= (1 << PD7);							// set PD7 as output for Gate LED
  DDRB = 0b11111111;              // PORTB output
  EIMSK |= (1<<INT0);			        // enable INT0
  EIMSK |= (1<<INT1);			        // enable INT1
  EICRA |= (1<<ISC00)|(1<<ISC01); // INT0 - Rising edge on INT0 generates an interrupt request
  EICRA |= (1<<ISC11);            // INT1 - Falling edge of INT1 generates interrupt request
  PORTB = 0b00000000;             // All outputs to zero
  count_steps = 0;
	max_steps = 0;
	ADC_Init();
  sei();			                    // Interrupts activate
  while (1)                       // for ever
    {				
			steps_read = ADC_Read(0);					// Steps pot read	
			if (steps_read <= 2)
				max_steps = 0;
			if ((steps_read > 2) && (steps_read <= 32))
				max_steps = 1;
			if ((steps_read > 32) && (steps_read <= 62))
				max_steps = 2;
			if ((steps_read > 62) && (steps_read <= 92))
				max_steps = 3;
			if ((steps_read > 92) && (steps_read <= 122))
				max_steps = 4;
			if ((steps_read > 122) && (steps_read <= 152))
				max_steps = 5;
			if ((steps_read > 152) && (steps_read <= 182))
				max_steps = 6;
			if ((steps_read > 182) && (steps_read <= 212))
				max_steps = 7;
			if (steps_read > 212)
				max_steps = 8;
    }
}
